# -*- coding: utf-8 -*-

# Songwrite 3
# Copyright (C) 2008-2016 Jean-Baptiste LAMY -- jibalamy@free.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import PyQt5.QtCore    as qtcore
import PyQt5.QtWidgets as qtwidgets
import PyQt5.QtGui     as qtgui

import songwrite3.model     as model
import songwrite3.canvas    as canvas_module



class LyreDrawer(canvas_module.TablatureDrawer):
  def __init__(self, canvas, partition, compact = False):
    canvas_module.TablatureDrawer.__init__(self, canvas, partition)
    self.compact = compact
    if compact: self.string_height = canvas.default_line_height / 1.5
    else:       self.string_height = 0.8 * canvas.default_line_height + 2 * self.scale
    self.note_offset_x =  0.66 * self.string_height
    self.note_offset_y = -10.0 * self.scale #-self.string_height // 22.0
    
  def mouse_motion_note(self, value, delta): return value + delta
  
  def note_listener(self, note, type, new, old):
    if type is object:
      if   (new.get("fx", "") == "harmonic") and (old.get("fx", "") != "harmonic"):
        string_value = self.strings[self.note_string_id(note)].base_note
        if abs(string_value - note.value) < 6: note.value += 12
        
      elif (new.get("fx", "") != "harmonic") and (old.get("fx", "") == "harmonic"):
        string_value = self.strings[self.note_string_id(note)].base_note
        if abs(string_value - note.value) >= 6: note.value -= 12
        
      elif new["value"] != old["value"]:
        string_value = self.strings[self.note_string_id(note)].base_note
        diff = abs(string_value - note.value)
        if   (diff >= 6) and (note.fx == ""):         note.set_fx("harmonic")
        elif (diff <  6) and (note.fx == "harmonic"): note.set_fx("")
        
    canvas_module.TablatureDrawer.note_listener(self, note, type, new, old)
    
  create_cursor = canvas_module.PartitionDrawer.create_cursor
  
  def note_text(self, note):
    if note.fx == "dead":
      return " "
    s = self.strings[self.note_string_id(note)].value_2_text(note)
    if (not note is self.canvas.cursor) and (s != "0") and (s != "12"): return s
    if note.fx == "harmonic":
      if note is self.canvas.cursor: return "_◆"
      return "◆"
    if note is self.canvas.cursor: return "_"
    return "●"
    
  def note_previous(self, note):
    string_id = self.note_string_id(note)
    while 1:
      note = note.previous()
      if not note: return None
      if self.note_string_id(note) == string_id: return note
      
  def draw_note(self, ctx, note, string_id, y):
    x      = self.canvas.time_2_x(note.time)
    string = self.strings[string_id]
    color  = self.note_color(note)
    
    ctx.save()
    ctx.setPen(color)
    ctx.setBrush(color)
    if   note.fx == "harmonic":
      if note.duration_fx == "appoggiatura": # White appoggiatura should never exist !
        ctx.drawPolygon(
          qtcore.QPointF(x + 0.35 * self.string_height, y),
          qtcore.QPointF(x + 0.6  * self.string_height, y + 0.25 * self.string_height),
          qtcore.QPointF(x + 0.85 * self.string_height, y),
          qtcore.QPointF(x + 0.6  * self.string_height, y - 0.25 * self.string_height),
        )
      else:
        ctx.drawPolygon(
          qtcore.QPointF(x + 0.2 * self.string_height, y),
          qtcore.QPointF(x + 0.6 * self.string_height, y + 0.4 * self.string_height),
          qtcore.QPointF(x + 1.0 * self.string_height, y),
          qtcore.QPointF(x + 0.6 * self.string_height, y - 0.4 * self.string_height),
        )
        if note.base_duration() in (192, 384):
          ctx.setBrush(qtcore.Qt.white)
          ctx.drawPolygon(
            qtcore.QPointF(x + 0.3 * self.string_height, y),
            qtcore.QPointF(x + 0.6 * self.string_height, y + 0.3 * self.string_height),
            qtcore.QPointF(x + 0.9 * self.string_height, y),
            qtcore.QPointF(x + 0.6 * self.string_height, y - 0.3 * self.string_height),
          )
          
    elif note.fx != "dead":
      if note.duration_fx == "appoggiatura": # White appoggiatura should never exist !
        ctx.drawEllipse(qtcore.QRectF(
          x + 0.35 * self.string_height,
          y - 0.25 * self.string_height + 0.5,
              0.5  * self.string_height,
              0.5  * self.string_height,
        ))
      else:
        ctx.drawEllipse(qtcore.QRectF(
          x + 0.2 * self.string_height,
          y - 0.4 * self.string_height + 0.5,
              0.8 * self.string_height,
              0.8 * self.string_height,
        ))
        if note.base_duration() in (192, 384): # White notes:
          ctx.setBrush(qtcore.Qt.white)
          ctx.drawEllipse(qtcore.QRectF(
            x + 0.3 * self.string_height,
            y - 0.3 * self.string_height + 0.5,
                0.6 * self.string_height,
                0.6 * self.string_height,
          ))
    ctx.restore()
    
    if note.base_duration() in (192, 384):
      line_y = self.string_id_2_y(string_id)
      self._draw_perfect_line(ctx, x + 0.2 * self.string_height, line_y, x + self.string_height, line_y)
      
    alteration = abs(note.value) - string.base_note
    if alteration > 6: alteration -= 12
    
    previous = self.note_previous(note)
    if previous and (self.partition.song.mesure_at(previous) is self.partition.song.mesure_at(note)):
      previous_alteration = abs(previous.value) - string.base_note
      if previous_alteration > 6: previous_alteration -= 12
    else: previous_alteration = 0
    
    if alteration != previous_alteration:
      if   alteration == -2: t = "♭♭"
      elif alteration == -1: t = "♭"
      elif alteration ==  0: t = "♮"
      elif alteration ==  1: t = "♯"
      elif alteration ==  2: t = "♯♯"
      else:                  t = "?"
      ctx.drawText(x - self.canvas.char_h_size * 0.5, y + self.canvas.default_ascent // 2 - 1, t)
      
    if note.is_dotted():
      #if string.line_style == 0: ctx.drawEllipse(x + 3.5 * self.string_height, y - 0.2 * self.string_height, 3 * self.scale, 3 * self.scale)
      ctx.drawEllipse(x + 1.3 * self.string_height, y - 0.5   * self.string_height, 3 * self.scale, 3 * self.scale)
      
    if note.fx     : getattr(self, "draw_note_fx_%s" % note.fx     )(ctx, note, string_id)
    if note.link_fx: getattr(self, "draw_note_fx_%s" % note.link_fx)(ctx, note, string_id)
    
  def note_width(self, note): return self.string_height
  
  on_touchscreen_new_note = canvas_module.PartitionDrawer.on_touchscreen_new_note
  
  def note_value_possible(self, note, value):
    for i, string in enumerate(self.strings): # Search a string with the right tone
      if ((string.base_note % 12) == (value % 12)): return 1
    return 0
  
  def note_string_id(self, note): return self.partition.view.note_string_id(note)
  
  def draw_stem_and_beam_for_chord(self, ctx, notes, notes_y, previous, next, appo = 0):
    if not appo:
      canvas_module.TablatureDrawer.draw_stem_and_beam_for_chord(self, ctx, notes, notes_y, previous, next, 0)
      
  _drawers                        = canvas_module.PartitionDrawer._drawers.copy()
  _drawers[288] = _drawers[192]   = canvas_module.PartitionDrawer._black
  _drawers8                       = canvas_module.PartitionDrawer._drawers8.copy()
  _drawers8[288] = _drawers8[192] = canvas_module.PartitionDrawer._black
  
  draw_note_fx_link = canvas_module.PartitionDrawer.draw_note_fx_link # No "p" / "h" for pull / hammer
  
  def draw(self, ctx, x, y, width, height, drag = 0):
    canvas_module.TablatureDrawer.draw(self, ctx, x, y, width, height, drag)
    if not drag:
      ctx.setFont(self.canvas.small_font)
      for i in range(len(self.strings)):
        label = model.note_label(self.strings[i].base_note, 0, self.partition.tonality)
        ctx.drawText(self.canvas.start_x - 2.2 * self.canvas.char_h_size, self.string_id_2_y(i) + self.canvas.small_ascent * 0.3, label)
      ctx.setFont(self.canvas.default_font)
      
    



# -*- coding: utf-8 -*-

# Songwrite 3
# Copyright (C) 2011 Jean-Baptiste LAMY -- jibalamy@free.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import editobj3
from editobj3.undoredo import *

import PyQt5.QtCore    as qtcore
import PyQt5.QtWidgets as qtwidgets
import PyQt5.QtGui     as qtgui

import songwrite3.model   as model
import songwrite3.globdef as globdef
import songwrite3.canvas  as canvas
import songwrite3.player  as player
from songwrite3.plugins.chord.chord import AccordionChordManager


class AccordionDrawer(canvas.TablatureDrawer):
  def __init__(self, canvas_, partition, compact = False):
    canvas.TablatureDrawer.__init__(self, canvas_, partition, compact)
    partition.g8 = 0
    self.chord_manager            = AccordionChordManager(partition)
    self.stem_extra_height_bottom = 0
    
  def drawers_changed(self):
    try:    next = self.canvas.drawers[self.canvas.drawers.index(self) + 1]
    except: self.right_hand_drawer = None
    else:
      if next.__class__.__name__ == "AccordionChordDrawer": self.right_hand_drawer = next
      else:                                                 self.right_hand_drawer = None
      
  def create_cursor(self, time, duration, string_id, value):
    cursor = canvas.PartitionDrawer.create_cursor(self, time, duration, string_id, value)
    if string_id == 0: cursor.bellows_dir = -1
    else:              cursor.bellows_dir =  1
    return cursor
  
  def on_key_press(self, event):
    if event.key() == qtcore.Qt.Key_Apostrophe: # '
      self.toggle_button_rank()
      return 1
    
  def toggle_button_rank(self):
    if not self.canvas.selections: return
    notes      = list(self.canvas.selections)
    old_values = {}
    for note in notes: old_values[note] = (note.value, note.button_rank)
    text = self.partition.view.strings[self.partition.view.note_string_id(notes[0])].value_2_text(notes[0])
    if "'" in text: new_button_rank = 0
    else:           new_button_rank = 1
    first = [1]
    def do_it():
      for note in notes:
        string = self.partition.view.strings[self.partition.view.note_string_id(note)]
        text   = string.value_2_text(note)
        if   (    "'" in text) and (new_button_rank == 0):
          note.value = string.text_2_value(note, text[:-1])
        elif (not "'" in text) and (new_button_rank == 1):
          note.value = string.text_2_value(note, text + "'")
        if first[0]:
          first[0] = 0
          player.play_note(note.partition.instrument, note.value)
          
    def undo_it():
      for note, (value, button_rank) in old_values.items():
        note.value = value
        if not button_rank is None: note.button_rank = button_rank
    UndoableOperation(do_it, undo_it, _("change button rank"), self.canvas.main.undo_stack)
    
  def note_value_possible(self, note, value):
    return value in self.partition.view.strings[self.partition.view.note_string_id(note)].value2buttons
  
  def draw(self, ctx, x, y, width, height, drag = 0, draw_icon = 1):
    canvas.TablatureDrawer.draw(self, ctx, x, y, width, height, drag, draw_icon)
    
  def draw_mesure(self, ctx, time, mesure, y, height):
    canvas.TablatureDrawer.draw_mesure(self, ctx, time, mesure, y - self.string_height // 2 - 1, height + self.string_height)
    
  def draw_strings(self, ctx, x, y, width, height):
    string_x  = x - 25.0 * self.scale
    string_x2 = x + width
    
    if (string_x < string_x2):
      #ctx.set_source_rgb(0.0, 0.0, 0.0)
      #ctx.set_line_width(1.0)
      for i in range(len(self.strings) + 1):
        string_y = self.string_id_2_y(i) - self.string_height // 2
        if i == 0: string_y0 = string_y
        if y - 2 < string_y < y + height + 2:
          string_y -= 0.5
          #ctx.move_to(string_x, string_y)
          #ctx.line_to(string_x2, string_y)
          #ctx.stroke()
          self._draw_perfect_line(ctx, string_x, string_y, string_x2, string_y)
      #ctx.move_to(string_x - 0.5, string_y  + 0.5)
      #ctx.line_to(string_x - 0.5, string_y0 - 1)
      #ctx.stroke()
      
      #self._draw_perfect_line(ctx, string_x - 0.5, string_y  + 0.5, string_x - 0.5, string_y0 - 1)
      self._draw_perfect_line(ctx, string_x, string_y, string_x, string_y0)
      
      #ctx.move_to(string_x + 8.0 * self.scale, string_y0 + self.string_height * 0.75)
      #ctx.show_text(_("__accordion__push"))
      #ctx.move_to(string_x + 8.0 * self.scale, string_y0 + self.string_height * 1.75)
      #ctx.show_text(_("__accordion__draw"))
      ctx.drawText(string_x + 8.0 * self.scale, string_y0 + self.string_height * 0.85, _("__accordion__push"))
      ctx.drawText(string_x + 8.0 * self.scale, string_y0 + self.string_height * 1.85, _("__accordion__draw"))
      
  def draw_note(self, ctx, note, string_id, y):
    if self.right_hand_drawer:
      right_hand_notes = self.right_hand_drawer.partition.notes_before(note.time + 1)
      while right_hand_notes:
        p = right_hand_notes[-1].next()
        if p and (p.time < note.end_time()):
          right_hand_notes.append(p)
        else: break
      times = {}
      for right_hand_note in right_hand_notes:
        if right_hand_note.end_time() > note.time:
          if right_hand_note.time in times: times[right_hand_note.time].append(right_hand_note)
          else:                             times[right_hand_note.time] =     [right_hand_note]
          
      ok = 1
      for time, right_hand_notes in times.items():
        if len(right_hand_notes) > 1: # chord
          rel_values = frozenset([right_hand_note.value for right_hand_note in right_hand_notes])
          chord      = self.right_hand_drawer.chord_manager.identify(rel_values)
          right_value = chord.base % 12
        else: # bass
          right_value = right_hand_notes[0].value % 12
          
        required_bellow_dir = CHORD_BELLOWS_DIR.get(right_value, 0)
        if required_bellow_dir and (note.bellows_dir != required_bellow_dir):
          x  = self.canvas.time_2_x(note.time)
          y += self.note_offset_y
          ctx.setPen(qtcore.Qt.red)
          ctx.drawText(x + 3 * self.scale, y + self.canvas.default_ascent / 2 - self.canvas.default_descent / 2, "X")
          break
        
    canvas.PartitionDrawer.draw_note(self, ctx, note, string_id, y)
    
  draw_note_fx_link = canvas.PartitionDrawer.draw_note_fx_link

CHORD_BELLOWS_DIR = {
   0 : -1, # Do
   2 :  1, # Ré
   4 : -1, # Mi
   5 :  0, # Fa
   7 :  0, # Sol
   9 :  1, # La
  }



# -*- coding: utf-8 -*-

# Songwrite 3
# Copyright (C) 2001-2016 Jean-Baptiste LAMY -- jibalamy@free.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# GlobDef : global stuff (cannot be called global nor glob so...).

import sys, os, os.path, gettext

APPDIR = os.path.dirname(__file__)

DATADIR = os.path.join(APPDIR, "data")
if not os.path.exists(DATADIR):
  import warnings
  warnings.warn("Songwrite3's data directory cannot be found !")
  
LOCALEDIR = os.path.join(APPDIR, "locale")
if not os.path.exists(LOCALEDIR):
  LOCALEDIR = os.path.join(APPDIR, "..", "locale")
  if not os.path.exists(LOCALEDIR):
    LOCALEDIR = os.path.join("/", "usr", "share", "locale")
    
try:
  translator = gettext.translation("songwrite3", LOCALEDIR)
except IOError: # Non-supported language, defaults to english
  translator = gettext.translation("songwrite3", LOCALEDIR, ("en",))
translator.install(1)

CONFIGFILE = os.path.expanduser(os.path.join("~", ".songwrite3"))
NO_CONFIG  = 0

class Config:
  def __init__(self):
    # Default config
    self.MIDI_COMMAND             = "timidity -idt -Wd -"
    self.MIDI_USE_TEMP_FILE       = 0
    self.PREVIEW_COMMAND_PDF      = ""
    self.PREVIEW_COMMAND_PDF2     = ""
    self.NUMBER_OF_UNDO           = 20
    self.PREVIOUS_FILES           = []
    self.PAGE_FORMAT              = "a4paper"
    self.PAGE_MARGIN_TOP          = 1.0
    self.PAGE_MARGIN_BOTTOM       = 2.0
    self.PAGE_MARGIN_INTERIOR     = 1.0
    self.PAGE_MARGIN_EXTERIOR     = 1.0
    self.DISPLAY_PLAY_BAR         = 1
    self.ASK_FILENAME_ON_EXPORT   = 1
    self.LAST_DIR                 = ""
    self.PLAY_AS_TYPING           = 1
    self.NUMBER_OF_PREVIOUS_FILES = 12
    self.AUTOMATIC_DURATION       = 1
    self.FONTSIZE                 = 0
    self.ENGLISH_CHORD_NAME       = 0
    self.WINDOW_X                 = -1 # -1 = auto
    self.WINDOW_Y                 = -1
    self.WINDOW_WIDTH             = 0 # 0 = maximized
    self.WINDOW_HEIGHT            = 0
    
    global NO_CONFIG
    if os.path.exists(CONFIGFILE):
      try:
        exec(compile(open(CONFIGFILE).read(), CONFIGFILE, 'exec'), self.__dict__)
        
        NO_CONFIG = 0
        return
      except:
        sys.excepthook(*sys.exc_info())
        print("Error in config file ~/.songwrite3 ! Please reconfigure Songwrite3 !", file = sys.stderr)
        
    NO_CONFIG = 1

  def get_preview_command_pdf(self):
    if not self.PREVIEW_COMMAND_PDF2:
      if self.PREVIEW_COMMAND_PDF:
        self.PREVIEW_COMMAND_PDF2 = self.PREVIEW_COMMAND_PDF
      else:
        from distutils.spawn import find_executable
        msg = _("Searching for PDF command: ")
        msgs = []
        for command in ["atril", "evince", "okular", "mupdf", "xpdf"]:
          path = find_executable(command)
          if path:
            self.PREVIEW_COMMAND_PDF2 = "%s %%s" % path
            msgs.append(command + _(": found"))
            break
          else:
            msgs.append(command + _(": not found"))
        print(msg + ", ".join(msgs), file = sys.stderr)
    return self.PREVIEW_COMMAND_PDF2
  
  def add_previous_file(self, file):
    try: self.PREVIOUS_FILES.remove(file) # No dupplicated item.
    except: pass
    self.PREVIOUS_FILES.insert(0, file)
    while len(self.PREVIOUS_FILES) > self.NUMBER_OF_PREVIOUS_FILES: del self.PREVIOUS_FILES[-1]
    
  def __str__(self):
    return """
# Songwrite config file. Use a Python syntax.

# 1 to use a temporary MIDI file (call this file %%s in the command line).
# 0 to use standard input.
MIDI_USE_TEMP_FILE = %s

# Command line to play a midi file.
MIDI_COMMAND = "%s"

# Page format (LaTeX)
PAGE_FORMAT = "%s"
PAGE_MARGIN_TOP      = %s
PAGE_MARGIN_BOTTOM   = %s
PAGE_MARGIN_INTERIOR = %s
PAGE_MARGIN_EXTERIOR = %s

# Command line to preview/print PDF (empty string for auto-detection).
PREVIEW_COMMAND_PDF = "%s"

# Size of undo stack.
NUMBER_OF_UNDO = %s

# Previous opened files.
PREVIOUS_FILES = %s

# Should the note currently being played be highlighted?
DISPLAY_PLAY_BAR = %s

# Should a "save as" dialog box be displayed before exporting?
ASK_FILENAME_ON_EXPORT = %s

# Last used directory
LAST_DIR = "%s"

# Play the note when they are entered
PLAY_AS_TYPING = %s

# Number of previous files in the file menu
NUMBER_OF_PREVIOUS_FILES = %s

# Automatically change previous note's duration when entering a new note
AUTOMATIC_DURATION = %s

# Interface font size (0 : auto)
FONTSIZE = %s

# Force English chord name
ENGLISH_CHORD_NAME = %s

# Default window pos (-1: auto)
WINDOW_X = %s
WINDOW_Y = %s

# Default window size (0: maximized)
WINDOW_WIDTH  = %s
WINDOW_HEIGHT = %s
""" % (self.MIDI_USE_TEMP_FILE, self.MIDI_COMMAND, self.PAGE_FORMAT, self.PAGE_MARGIN_TOP, self.PAGE_MARGIN_BOTTOM, self.PAGE_MARGIN_INTERIOR, self.PAGE_MARGIN_EXTERIOR, self.PREVIEW_COMMAND_PDF, self.NUMBER_OF_UNDO, repr(self.PREVIOUS_FILES), self.DISPLAY_PLAY_BAR, self.ASK_FILENAME_ON_EXPORT, self.LAST_DIR, self.PLAY_AS_TYPING, self.NUMBER_OF_PREVIOUS_FILES, self.AUTOMATIC_DURATION, self.FONTSIZE, self.ENGLISH_CHORD_NAME, self.WINDOW_X, self.WINDOW_Y, self.WINDOW_WIDTH, self.WINDOW_HEIGHT)

  def save(self):
    open(CONFIGFILE, "w").write(str(self))

  def edit(self):
    import songwrite3.__editobj3__, editobj3
    
    def on_validate(obj):
      self.MIDI_USE_TEMP_FILE = self.MIDI_COMMAND.find("%s") != -1
      self.save()
      
    editobj3.edit(self, on_validate)
    
    
config = Config()

